@@ -3,8 +3,10 @@ from typing import Callable
 from returns.maybe import Maybe, Nothing
 
 from pinjected import Injected
-from pinjected.di.bindings import BindMetadata, InjectedBind
 from pinjected.di.injected import PartialInjectedFunction, InjectedFunction
+from pinjected.di.metadata.bind_metadata import BindMetadata
+from pinjected.v2.binds import BindInjected
+from pinjected.v2.keys import StrBindKey
 
 
 def update_if_registered(
@@ -17,10 +19,10 @@ def update_if_registered(
         case PartialInjectedFunction(InjectedFunction(tgt, mapping)):
             res = PartialInjectedFunction(updated)
             from pinjected.di.implicit_globals import IMPLICIT_BINDINGS
-            key = binding_key if binding_key is not None else tgt.__name__
-            IMPLICIT_BINDINGS[key] = InjectedBind(
+            key = StrBindKey(binding_key) if binding_key is not None else StrBindKey(tgt.__name__)
+            IMPLICIT_BINDINGS[key] = BindInjected(
                 res,
-                metadata=meta
+                _metadata=meta
             )
             return res
         case _:
