@@ -17,15 +17,16 @@ from rich.panel import Panel
 
 from pinjected.di.app_injected import EvaledInjected
 from pinjected.di.ast import Expr
-from pinjected.di.bindings import Bind, BindMetadata
 from pinjected.di.designed import Designed
 from pinjected.di.injected import Injected, InjectedByName, InjectedFunction
+from pinjected.di.metadata.bind_metadata import BindMetadata
 from pinjected.di.metadata.location_data import ModuleVarLocation
 from pinjected.di.proxiable import DelegatedVar
 from pinjected.di.sessioned import Sessioned
 from pinjected.exceptions import DependencyResolutionFailure, DependencyResolutionError
 from pinjected.graph_inspection import DIGraphHelper
 from pinjected.providable import Providable
+from pinjected.v2.binds import IBind
 
 T = TypeVar("T")
 
@@ -341,16 +342,24 @@ class DependencyResolver:
 
         self.memoized_deps = _memoized_deps
 
-    def _dfs(self, tgt: str, visited: set[str] = None, include_dynamic=False):
+    def _dfs(self, tgt: str,trace:list[str]=None, visited: set[str] = None, include_dynamic=False):
         if visited is None:
             visited = set()
         if tgt in visited:
             return
+        if trace is None:
+            trace = []
+        trace += [tgt]
         visited.add(tgt)
-        deps = self.memoized_deps(tgt, include_dynamic=include_dynamic)
+        try:
+            deps = self.memoized_deps(tgt, include_dynamic=include_dynamic)
+        except NoMappingError as ke:
+            from loguru import logger
+            logger.error(f"failed to find dependency for {tgt} in {' -> '.join(trace)}")
+            raise NoMappingError(f"failed to find dependency for {tgt} in {' -> '.join(trace)}") from ke
         yield tgt
         for dep in deps:
-            yield from self._dfs(dep, visited, include_dynamic=include_dynamic)
+            yield from self._dfs(dep,trace, visited, include_dynamic=include_dynamic)
 
     def required_dependencies(self, providable: Providable, include_dynamic=False) -> Set[str]:
         tgt: Injected = self._to_injected(providable)
@@ -488,7 +497,7 @@ class DependencyResolver:
                     try:
                         return provider(*resolved_deps)
                     except Exception as e:
-                        bind: Bind = self.helper.total_bindings()[current_tgt]
+                        bind: IBind = self.helper.total_bindings()[current_tgt]
                         match bind.metadata:
                             case Some(BindMetadata(code_location=Some(ModuleVarLocation(path, line, column)))):
                                 location = f"{path}:{line}:{column}"
