@@ -15,6 +15,7 @@ from returns.result import safe
 
 from pinjected.di.injected_analysis import get_instance_origin
 from pinjected.di.proxiable import DelegatedVar
+from pinjected.v2.ainjected import MappedAInjected
 
 T, U = TypeVar("T"), TypeVar("U")
 
@@ -93,6 +94,22 @@ class ParamInfo:
     # the problem is we may or may not have vargs and kwargs...
 
 
+"""
+Semantics of an Injected:
+For Injected[T], upon resolving T, all dependencies are resolved for T.
+So, if T is a Callable, then any dependencies are resolved before calling it.
+
+However, for Lazy Injected thing, we want the dependencies to be resolved inside the function.
+This is because,, for asyncio stuff, the dependencies end their life after asyncio.run has finished for resolving the dependencies.
+For asyncio thing to work, we need to keep asyncio.run unfinished and only run at the top level for once.
+So, the solutions are:
+1. make async functions resolve dependencies on demand.
+2. make run_injected async and call asyncio.run only once.
+
+Let's got with 2nd option.
+"""
+
+
 class Injected(Generic[T], metaclass=abc.ABCMeta):
     """
     The ``Injected`` class represents a sophisticated dependency injection mechanism in Python.
@@ -243,9 +260,10 @@ class Injected(Generic[T], metaclass=abc.ABCMeta):
             new_func_sig = f"injected_{funcname}({','.join([str(p).split(':')[0] for p in (missing_non_defaults + varg + vkwarg)])})"
             return new_func_sig
 
-        def makefun_impl(injected_kwargs):
+        async def makefun_impl(injected_kwargs):
             # this gets called every time you call this function through PartialInjectedFunction interface
             # this is because the injected_kwargs gets changed.
+            assert isinstance(injected_kwargs, dict), f"expected dict, got {injected_kwargs}"
 
             missing_keys = [k for k in original_sig.parameters.keys() if k not in injected_kwargs]
             missing_params = [original_sig.parameters[k] for k in missing_keys]
@@ -268,6 +286,7 @@ class Injected(Generic[T], metaclass=abc.ABCMeta):
                 # logger.info(f"inferred positional arg names:{inferred_pos_arg_names}")
                 # logger.info(f"inferred non positional arg names:{inferred_non_pos_arg_names}")
                 total_kwargs = copy(defaults)
+                # logger.info(f"injected_kwargs:{injected_kwargs}")
                 filled_kwargs = {**injected_kwargs,
                                  **dict(zip(inferred_pos_arg_names, inferred_positional_args)),
                                  **dict(zip(inferred_non_pos_arg_names, inferred_non_positional_args)),
@@ -314,13 +333,15 @@ class Injected(Generic[T], metaclass=abc.ABCMeta):
             )
             new_func.__is_async__ = inspect.iscoroutinefunction(original_function)
             __doc__ = original_function.__doc__
-            __skeleton__ = f"""def {new_func_sig}:
+            __skeleton__ = f"""async def {new_func_sig}:
     \"\"\"
     {__doc__}
     \"\"\"
 """
             new_func.__skeleton__ = __skeleton__
 
+            #logger.info(f"result of makefun_impl:{new_func}")
+
             return new_func
 
         makefun_impl.__name__ = original_function.__name__
@@ -338,15 +359,18 @@ class Injected(Generic[T], metaclass=abc.ABCMeta):
             makefun_impl.__original_file__ = safe(inspect.getfile)(original_function).value_or("not available")
 
         makefun_impl.__doc__ = original_function.__doc__
+        #logger.info(f"injection_targets:{injection_targets}")
         injected_kwargs = Injected.dict(**injection_targets)
+        #logger.info(f"injected_kwargs:{injected_kwargs}")
         # hmm?
         # Ah, so upon calling instance.method(), we need to manually check if __self__ is present?
-        injected_factory = PartialInjectedFunction(
-            Injected.bind(makefun_impl, injected_kwargs=injected_kwargs)
-        )
+        bound_func = Injected.bind(makefun_impl, injected_kwargs=injected_kwargs)
+        bound_func.__is_async_function__ = inspect.iscoroutinefunction(original_function)
+        injected_factory = PartialInjectedFunction(src=bound_func)
         # the inner will be called upon calling the injection result.
         # This involves many internal Injecte instances. can I make it simler?
         # it takes *by_name, mzip, and map.
+        injected_factory.__is_async_function__ = bound_func.__is_async_function__
         injected_factory.__runnable_metadata__ = {
             "kind": "callable"
         }
@@ -366,10 +390,23 @@ class Injected(Generic[T], metaclass=abc.ABCMeta):
 
     @staticmethod
     def bind(_target_function_, **kwargs_mapping: Union[str, type, Callable, "Injected"]) -> "InjectedFunction":
-        if isinstance(_target_function_, Injected):
-            _target_function_ = _target_function_.get_provider()
+        assert not isinstance(_target_function_, Injected)
+        # if isinstance(_target_function_, Injected):
+        #     _target_function_ = _target_function_.get_provider()
+        assert callable(_target_function_)
+        if not inspect.iscoroutinefunction(_target_function_):
+            # we need to keep the function signature
+
+            async def _a_target_function(*args, **kwargs):
+                return _target_function_(*args, **kwargs)
+
+            _a_target_function.__signature__ = inspect.signature(_target_function_)
+
+            async_target_function = _a_target_function
+        else:
+            async_target_function = _target_function_
         return InjectedFunction(
-            target_function=_target_function_,
+            target_function=async_target_function,
             kwargs_mapping=kwargs_mapping
         )
 
@@ -439,7 +476,16 @@ class Injected(Generic[T], metaclass=abc.ABCMeta):
         """
 
     def map(self, f: Callable[[T], U]) -> 'Injected[U]':
-        return MappedInjected(self, f)
+        # return MappedInjected(self, f)
+        if not inspect.iscoroutinefunction(f):
+
+            async def async_f(*args, **kwargs):
+                return f(*args, **kwargs)
+
+            new_f = async_f
+        else:
+            new_f = f
+        return MappedInjected(self, new_f)
 
     def from_impl(impl: Callable, dependencies: Set[str]):
         return GeneratedInjected(impl, dependencies)
@@ -455,7 +501,7 @@ class Injected(Generic[T], metaclass=abc.ABCMeta):
     def zip(self, other: "Injected[U]") -> "Injected[Tuple[T,U]]":
         assert isinstance(self, Injected)
         assert isinstance(other, Injected)
-        return ZippedInjected(self, other)
+        return Injected.mzip(self, other)
 
     @staticmethod
     def mzip(*srcs: "Injected"):
@@ -467,7 +513,7 @@ class Injected(Generic[T], metaclass=abc.ABCMeta):
         return Injected.mzip(*srcs).map(lambda t: tuple(t))
 
     @staticmethod
-    def list(*srcs: "Injected"):
+    def list(*srcs: Union["Injected", "DelegatedVar"]):
         return Injected.mzip(*srcs).map(list)
 
     @staticmethod
@@ -510,7 +556,9 @@ class Injected(Generic[T], metaclass=abc.ABCMeta):
 
     @staticmethod
     def dict(**kwargs: "Injected") -> "Injected[Dict]":
+        # raise RuntimeError("disabled")
         keys = list(kwargs.keys())
+
         return Injected.mzip(*[kwargs[k] for k in keys]).map(lambda t: {k: v for k, v in zip(keys, t)})
 
     @property
@@ -539,10 +587,12 @@ class Injected(Generic[T], metaclass=abc.ABCMeta):
             case int() | float() | str():
                 other = Injected.pure(other)
         other = Injected.ensure_injected(other)
-        return self.zip(other).map(lambda t: t[0] + t[1])
+        return (self.proxy + other).eval()
+        # return self.zip(other).map(lambda t: t[0] + t[1])
 
     def __getitem__(self, item):
-        return self.map(lambda x: x[item])
+        # return self.map(lambda x: x[item])
+        return self.proxy[item]
 
     def desync(self):
         async def impl(awaitable):
@@ -550,7 +600,7 @@ class Injected(Generic[T], metaclass=abc.ABCMeta):
 
         return self.map(lambda coroutine: asyncio.run(impl(coroutine)))
 
-    def add_dynamic_dependencies(self, *deps: Union[str,set[str]]):
+    def add_dynamic_dependencies(self, *deps: Union[str, set[str]]):
         deps_set = set()
         for item in deps:
             match item:
@@ -873,20 +923,27 @@ class InjectedWithDynamicDependencies(Injected[T]):
 class MappedInjected(Injected):
     __match_args__ = ("src", "f")
 
-    def __init__(self, src: Injected[T], f: Callable[[T], U]):
+    def __init__(self, src: Injected[T], f: Callable[[T], Awaitable[U]]):
         super(MappedInjected, self).__init__()
+        assert inspect.iscoroutinefunction(f), f"{f} is not a coroutine function"
         self.src = src
-        self.f: Callable[[T], U] = f
+        self.f: Callable[[T], Awaitable[U]] = f
 
     def dependencies(self) -> Set[str]:
         return self.src.dependencies()
 
     def get_provider(self):
-        def impl(**kwargs):
+        async def impl(**kwargs):
             assert self.dependencies() == set(
                 kwargs.keys())  # this is fine but get_provider returns wrong signatured func
-            tmp = self.src.get_provider()(**kwargs)
-            return self.f(tmp)
+            if not inspect.iscoroutinefunction(self.src.get_provider()):
+                raise RuntimeError(f"provider is not a corountine function:{self.src}")
+            tmp = await self.src.get_provider()(**kwargs)
+            # logger.info(f"original result:{tmp}")
+            # logger.info(f"mapper function:{self.f}")
+            data = await self.f(tmp)
+            # logger.info(f"mapped result:{data}")
+            return data
 
         return create_function(self.get_signature(), func_impl=impl)
 
@@ -928,15 +985,15 @@ def extract_dependency(dep: Union[str, type, Callable, Injected, DelegatedVar[In
         raise RuntimeError(f"dep must be either str/type/Callable/Injected. got {type(dep)}")
 
 
-def solve_injection(dep: Union[str, type, Callable, Injected], kwargs: dict):
+async def solve_injection(dep: Union[str, type, Callable, Injected], kwargs: dict):
     if isinstance(dep, str):
         return kwargs[dep]
     elif isinstance(dep, DelegatedVar):
-        return solve_injection(dep.eval(), kwargs)
+        return await solve_injection(dep.eval(), kwargs)
     elif isinstance(dep, Injected):
-        return solve_injection(dep.get_provider(), kwargs)
-    elif isinstance(dep, (type, Callable)):
-        return dep(**{k: kwargs[k] for k in extract_dependency(dep)})
+        return await solve_injection(dep.get_provider(), kwargs)
+    elif isinstance(dep, (type, Callable)) and inspect.iscoroutinefunction(dep):
+        return await dep(**{k: kwargs[k] for k in extract_dependency(dep)})
     else:
         raise RuntimeError(f"dep must be one of str/type/Callable/Injected. got {type(dep)}")
 
@@ -973,6 +1030,7 @@ class InjectedFunction(Injected[T]):
         assert not isinstance(target_function, (Injected, DelegatedVar))
         assert callable(target_function)
         self.target_function = target_function
+        assert inspect.iscoroutinefunction(self.target_function), f"{self.target_function} is not a coroutine function"
         self.kwargs_mapping = copy(kwargs_mapping)
         for k, v in self.kwargs_mapping.items():
             assert_kwargs_type(v)
@@ -980,9 +1038,9 @@ class InjectedFunction(Injected[T]):
                 self.kwargs_mapping[k] = v.eval()
         # logger.info(f"InjectedFunction:{self.target_function} kwargs_mapping:{self.kwargs_mapping}")
         org_deps = extract_dependency(self.target_function)
-        # logger.info(f"tgt:{target_function} original dependency:{org_deps}")
+        logger.trace(f"tgt:{target_function} original dependency:{org_deps}")
         missings = {d for d in org_deps if d not in self.kwargs_mapping}
-        # logger.info(f"now missing {missings}")
+        logger.trace(f"now missing {missings}")
         # logger.warning(f"created InjectedFunction:{inspect.signature(target_function)}")
         # assert "self" not in inspect.signature(target_function).parameters
         self.missings = missings
@@ -993,16 +1051,32 @@ class InjectedFunction(Injected[T]):
     def get_provider(self):
         signature = self.get_signature()
 
-        def impl(**kwargs):
+        async def impl(**kwargs):
             deps = dict()
+
+            async def update(key):
+                if key not in deps:
+                    if key in self.kwargs_mapping:
+                        mapped = self.kwargs_mapping[key]
+                    else:
+                        mapped = key
+                    deps[key] = await solve_injection(mapped, kwargs)
+
+            tasks = []
+            logger.trace(f"missings:{self.missings},kwargs_mapping:{self.kwargs_mapping}")
             for mdep in self.missings:
-                deps[mdep] = solve_injection(mdep, kwargs)
+                tasks.append(update(mdep))
             for k, dep in self.kwargs_mapping.items():
-                deps[k] = solve_injection(dep, kwargs)
+                tasks.append(update(k))
+            await asyncio.gather(*tasks)
             # logger.info(f"calling function:{self.target_function.__name__}{inspect.signature(self.target_function)}")
             # logger.info(f"src mapping:{self.kwargs_mapping}")
             # logger.info(f"with deps:{deps}")
-            return self.target_function(**deps)
+            logger.trace(f"awaiting target function:{self.target_function}")
+            logger.trace(f"deps:{deps}")
+            res = await self.target_function(**deps)
+            # assert not inspect.iscoroutinefunction(res),f"result of awaiting {self.target_function} is a coroutine function:{res}"
+            return res
 
         # you have to add a prefix 'provider'""
         return create_function(func_signature=signature, func_impl=impl)
@@ -1038,7 +1112,10 @@ class InjectedPure(Injected[T]):
         return set()
 
     def get_provider(self):
-        return create_function(func_signature=self.get_signature(), func_impl=lambda: self.value)
+        async def impl():
+            return self.value
+
+        return create_function(func_signature=self.get_signature(), func_impl=impl)
 
     def __str__(self):
         return f"Pure({self.value})"
@@ -1061,7 +1138,10 @@ class InjectedByName(Injected[T]):
         return {self.name}
 
     def get_provider(self):
-        return create_function(func_signature=self.get_signature(), func_impl=lambda **kwargs: kwargs[self.name])
+        async def impl(**kwargs):
+            return kwargs[self.name]
+
+        return create_function(func_signature=self.get_signature(), func_impl=impl)
 
     def __str__(self):
         return f"InjectedByName({self.name})"
@@ -1078,6 +1158,7 @@ class ZippedInjected(Injected[Tuple[A, B]]):
 
     def __init__(self, a: Injected[A], b: Injected[B]):
         super().__init__()
+        raise RuntimeError("deprecated")
         assert isinstance(a, Injected), f"got {type(a)} for a"
         assert isinstance(b, Injected), f"got {type(b)} for b"
         self.a = a
@@ -1118,17 +1199,16 @@ class MZippedInjected(Injected):
         return res
 
     def get_provider(self):
-        def impl(**kwargs):  # can we pickle this though?
-            each_kwargs = [{k: kwargs[k] for k in s.dependencies()} for s in self.srcs]
+        async def impl(**kwargs):  # can we pickle this though?
             res = []
             for s in self.srcs:
                 r = s.get_provider()(**{k: kwargs[k] for k in s.dependencies()})
+                if not inspect.iscoroutinefunction(s.get_provider()):
+                    raise RuntimeError(f"provider is not a corountine function:{s}")
                 res.append(r)
-            return tuple(res)
+            return tuple(await asyncio.gather(*res))
 
         signature = self.get_signature()
-        # from loguru import logger
-        # logger.info(f"created signature:{signature} for MZippedInjected")
         return create_function(func_signature=signature, func_impl=impl)
 
     def dynamic_dependencies(self) -> Set[str]:
@@ -1184,12 +1264,25 @@ class RunnableInjected(Injected):
 @dataclass
 class PartialInjectedFunction(Injected):
     src: Injected[Callable]
+    """ 
+    here the src is Awaitable[Callable]. so..
+    we get T->Awaitable[U] after resolving it.
+    so, (T->Awaitable[U])(T) == Awaitable[U].
+    Therefore we need to perform 'awaitcall' rather than 'call' here.
+    
+    """
 
     def __post_init__(self):
         assert isinstance(self.src, Injected), f"src:{self.src} is not an Injected"
 
     def __call__(self, *args, **kwargs) -> DelegatedVar:
-        return self.src.proxy(*args, **kwargs)
+        """
+        we need 'await' call here.
+        So, we need Await AST too.
+        """
+
+        res = self.src.proxy(*args, **kwargs)
+        return res
 
     def dependencies(self) -> Set[str]:
         return self.src.dependencies()
