@@ -12,7 +12,6 @@ from loguru import logger
 from returns.pipeline import is_successful
 from returns.result import safe, Result, Failure
 
-from pinjected.di.bindings import InjectedBind
 from pinjected.di.graph import providable_to_injected
 from pinjected.providable import Providable
 from pinjected.di.injected import Injected, InjectedFunction, InjectedPure, MappedInjected, \
@@ -24,6 +23,7 @@ from pinjected.exceptions import DependencyResolutionFailure, _MissingDepsError
 from pinjected.graph_inspection import DIGraphHelper
 from pinjected.module_var_path import ModuleVarPath
 from pinjected.nx_graph_util import NxGraphUtil
+from pinjected.v2.binds import BindInjected
 
 
 def dfs(neighbors: Callable, node: str, trace=[]):
@@ -63,13 +63,8 @@ class DIGraph:
         return f"{base}_{str(uuid.uuid4())[:6]}"
 
     def __post_init__(self):
-        self.src = self.src.bind_instance(session='DummyForVisualization').build()
         self.helper = DIGraphHelper(self.src)
-        # TODO combine these mappings into one. the keys must not override each other
-        self.implicit_mappings = dict(self.helper.get_implicit_mapping())
-        self.explicit_mappings: Dict[str, Injected] = {k: b.to_injected() for k, b in self.src.bindings.items()}
-        self.explicit_mappings.update(**self.helper.total_mappings())
-        self.multi_mappings = {k: b for k, b in self.src.multi_binds.items()}
+        self.explicit_mappings:dict[str,Injected] = self.helper.total_mappings()
 
         self.direct_injected = dict()
         self.injected_to_id = dict()
@@ -81,12 +76,6 @@ class DIGraph:
             if src in self.explicit_mappings:
                 em = self.explicit_mappings[src]
                 return self.resolve_injected(em)
-            elif src in self.implicit_mappings:
-                bind = Injected.bind(self.implicit_mappings[src])
-                return bind.complete_dependencies
-            elif src in self.multi_mappings:
-                return list(
-                    set(chain(*[Injected.bind(tgt).complete_dependencies for tgt in self.multi_mappings[src]])))
             elif src in self.direct_injected:
                 di = self.direct_injected[src]
                 return self.resolve_injected(di)
@@ -151,10 +140,6 @@ class DIGraph:
             self.src.bindings, key
         ).lash(
             lambda e: getitem(self.explicit_mappings, key)
-        ).lash(
-            lambda e: getitem(self.implicit_mappings, key)
-        ).lash(
-            lambda e: getitem(self.multi_mappings, key)
         ).lash(
             lambda e: getitem(self.direct_injected, key)
         )
@@ -167,7 +152,6 @@ class DIGraph:
 
     def di_dfs(self, src, replace_missing=False):
         ignore_list = ["mzip_src_", "mapped_src_", "injected_kwargs_"]
-        #ignore_list = []
 
         def filter(node):
             res = any([ignore in node for ignore in ignore_list])
@@ -233,7 +217,6 @@ class DIGraph:
                 )
                 return tmp_design.to_vis_graph().distilled("__target__")
 
-
     @staticmethod
     def get_source(f):
         try:
@@ -251,6 +234,12 @@ class DIGraph:
             from loguru import logger
             logger.warning(f"{repr(e)[:100]} error from {repr(f)[:100]}")
             res = f"failed:{f} from {f}"
+        return res
+
+
+    @staticmethod
+    def get_source_repr(f):
+        res = DIGraph.get_source(f)
 
         res = res.replace("<", "").replace(">", "")
         res = res.replace("\t", "....").replace(" ", ".")
@@ -264,19 +253,19 @@ class DIGraph:
                 return self.parse_injected(src)
             case InjectedFunction(f):
                 desc = f"Injected:{safe(getattr)(f, '__name__').value_or(repr(f))}"
-                return ("injected", desc, self.get_source(f))
+                return ("injected", desc, self.get_source_repr(f))
             case InjectedPure(v):
                 desc = f"Pure:{v}"
-                return ("injected", desc, self.get_source(v) if isinstance(v, Callable) else str(v))
+                return ("injected", desc, self.get_source_repr(v) if isinstance(v, Callable) else str(v))
             case PartialInjectedFunction(InjectedFunction(src)):
                 desc = f"partial=>{src.__name__}"
-                return ("injected", desc, self.get_source(src))
+                return ("injected", desc, self.get_source_repr(src))
             case PartialInjectedFunction(src):
                 desc = f"partial=>{src}"
-                return ("injected", desc, self.get_source(src))
+                return ("injected", desc, self.get_source_repr(src))
             case MappedInjected(src, mapping) as mi:
                 desc = f"{mi.__class__.__name__}"
-                return ("injected", desc, self.get_source(mapping))
+                return ("injected", desc, self.get_source_repr(mapping))
             case ZippedInjected(srcs) as zi:
                 desc = f"{zi.__class__.__name__}"
                 return ("injected", desc, "zipped")
@@ -321,10 +310,10 @@ class DIGraph:
                 match tgt:
                     case Injected():
                         return self.parse_injected(tgt)
-                    case InjectedBind(tgt):
+                    case BindInjected(tgt,_):
                         return self.parse_injected(tgt)
                     case cls if isinstance(cls, type):
-                        return ("class", cls.__name__, self.get_source(cls))
+                        return ("class", cls.__name__, self.get_source_repr(cls))
                     case [*providers]:
                         return ("multi_binding", repr(providers), repr(providers))
                     case _:
@@ -502,6 +491,7 @@ g = d.to_graph()
         self.create_dependency_digraph(roots, replace_missing=True, root_group=None).show_html_temp()
 
 
+
 def create_dependency_graph(d: Design, roots: List[str], output_file="dependencies.html"):
     dig = DIGraph(d.bind_instance(
         job_type="net_visualization"
